package edu.umd.dawn.common.jwt;

import com.auth0.jwt.JWT;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.interfaces.DecodedJWT;
import edu.umd.dawn.common.exceptions.BaseExceptions;
import edu.umd.dawn.common.exceptions.DawnException;
import lombok.Getter;

@Getter
public class JWTUtil {

    private Algorithm algorithm;
    private String accessSecret;
    private String jwtString;
    private DecodedJWT jwt;

    public JWTUtil(String accessSecret, String jwt) {
        this.accessSecret = accessSecret;
        this.jwtString = jwt;
        initAlgorithm();
        decode();
    }

    private void initAlgorithm() {
        this.algorithm = Algorithm.HMAC256(accessSecret);
    }

    private void decode() {
        try {
            this.jwt = JWT.require(algorithm)
                    .acceptLeeway(1) // 1 sec for nbf and iat
                    .acceptExpiresAt(5) // 5 secs for exp
                    .withClaimPresence("id") // need id in the claim
                    .build()
                    .verify(jwtString);
        } catch (Exception e) {
            throw new DawnException(BaseExceptions.INVALID_JWT, e);
        }
    }

    public Claims getClaims() {
        return Claims.build(this);
    }

    public static JWTUtil parse(String accessSecret, String jwt) {
        return new JWTUtil(accessSecret, jwt);
    }
}
