package edu.umd.dawn.common.logging;

import java.net.InetAddress;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;
import org.apache.logging.log4j.core.LogEvent;
import org.apache.logging.log4j.message.ObjectMessage;
import org.slf4j.MDC;
import org.springframework.stereotype.Component;

@Component
@Data
@SuperBuilder
@AllArgsConstructor
@NoArgsConstructor
public class Log {

    private String requestId;
    private String time;
    private long thread;
    private String service;
    private String host;

    public void fillSharedFields(LogEvent event) {
        requestId = MDC.get("requestId");
        service = MDC.get("serviceName");
        time = LocalDateTime.ofInstant(Instant.ofEpochMilli(event.getTimeMillis()), ZoneId.systemDefault())
                .toString();
        thread = event.getThreadId();
        try {
            host = InetAddress.getLocalHost().toString();
        } catch (Exception e) {
            host = "";
        }
    }

    public ObjectMessage toObjectMessage() {
        return new ObjectMessage(this);
    }
}
